@@ -16,7 +16,6 @@
 #define TRUE	1
 #endif
 
-#define PNG_INTERNAL
 #include "gif2png.h"
 
 /* Define png_jmpbuf() in case we are using a pre-1.0.6 version of libpng */
@@ -129,13 +128,13 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
     png_byte pal_trans[MAXCMSIZE];
     png_color_16 color16trans, color16back;
     byte buffer[24]; /* used for gIFt and gIFg */
-    byte *data;
     int j;
     png_uint_16 histogr[MAXCMSIZE];
     unsigned long hist_maxvalue;
     int passcount;
     int errtype, errorcount = 0;
     png_text software;
+    png_text comment;
 
     /* these volatile declarations prevent gcc warnings ("variable might be
      *  clobbered by `longjmp' or `vfork'") */
@@ -369,8 +368,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 		    if (last_color < i) 
 			last_color = i;
 		    color16back.index = i;
-		    info_ptr->background = color16back;
-		    info_ptr->valid |= PNG_INFO_bKGD;
+		    png_set_bKGD(png_ptr, info_ptr, &color16back);
 		    break;
 		}
 	    }
@@ -387,13 +385,7 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	} else {
 	    if (verbose > 2)
 		fprintf(stderr, "gif2png: handling palette transparency\n");
-#if 0
-	    for (i = 0; i < MAXCMSIZE; i++)
-		pal_trans[i] = 255;
-	    pal_trans[img->trans] = 0;
-	    png_set_tRNS(png_ptr, info_ptr, pal_trans, img->trans+1, NULL);
-#else
-	    /* GRR NOTE:  may be inconsistent with background-color logic (above) */
+	    /* GRR NOTE: may be inconsistent with background-color logic (above) */
 	    remapping = TRUE;
 	    for (i = 0; i < MAXCMSIZE; i++)
 		remap[i] = i;
@@ -401,7 +393,6 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 	    remap[img->trans] = 0;
 	    pal_trans[0] = 0;
 	    png_set_tRNS(png_ptr, info_ptr, pal_trans, 1, NULL);
-#endif
 	}
     }
 
@@ -473,13 +464,14 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 
     png_write_info(png_ptr, info_ptr);
 
-    if (info_ptr->bit_depth<8)
+    if (bitdepth<8)
 	png_set_packing(png_ptr);
 
     /* loop over elements until we reach the image or the last element if
        this is the last image in a GIF */
 
     while(lastimg ? s != NULL : s != e->next) {
+	byte *data;
 	switch((byte)s->GIFtype) {
 	case GIFimage:
 	    passcount = png_set_interlace_handling(png_ptr);
@@ -492,42 +484,39 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 				(int)(((long)i*100)/current->imagestruct->height));
 			fflush(stderr);
 		    }
-		    data = access_data(current, (long) (img->interlace ?
-							interlace_line(current->imagestruct->height,i) : i) *
-				       current->imagestruct->width,
-				       current->imagestruct->width);
-#ifndef TMPFILE
-		    /* if we store the image in memory, we have to remap once */
+		    data = current->data + 
+				       (long) (img->interlace ?
+							interlace_line(current->imagestruct->height,i) : i) * current->imagestruct->width;
+		    /* we have to remap once */
 		    if (remapping && p == 0)
-#else
-			/* if we store the image in a file, we have to remap each time */
-			if (remapping)
-#endif
-			{
-			    for (j = 0; j < img->width; j++) {
-				data[j] = remap[data[j]];
-			    }
+		    {
+			for (j = 0; j < img->width; j++) {
+			    data[j] = remap[data[j]];
 			}
+		    }
 		    png_write_row(png_ptr, data);
 		}
 	    break;
 
 	case GIFcomment:
-	    data = access_data(s, 0, s->size);
 	    j = s->size;
-	    if (j > 0 && data[j-1] == '\0')  /* some apps include a NULL in GIF comment */
+	    /* some GIF encoders produce trailing garbage and NULs */
+	    if (j > 0 && ((s->data[j-1] == '\0') || (s->data[j-1] & 0x80)))
 		--j;
+	    comment.key = "Comment";
 	    if (j<500) {
-		png_write_tEXt(png_ptr, "Comment", (png_charp)data, j);
+		comment.compression = PNG_TEXT_COMPRESSION_NONE;
 	    } else {
-		png_write_zTXt(png_ptr, "Comment", (png_charp)data, j, 0);
+		comment.compression = PNG_TEXT_COMPRESSION_zTXt;
 	    }
+ 	    comment.text = (png_charp)s->data;
+ 	    comment.text_length = strlen(comment.text);
+ 	    png_set_text(png_ptr, info_ptr, &comment, 1);
 	    break;
 
 	case GIFapplication:
-	    data = access_data(s, 0, s->size);
 	    png_write_chunk_start(png_ptr, (unsigned char *)"gIFx", s->size);
-	    png_write_chunk_data(png_ptr, data, s->size);
+	    png_write_chunk_data(png_ptr, s->data, s->size);
 	    png_write_chunk_end(png_ptr);
 	    break;
 
@@ -537,39 +526,37 @@ int writefile(struct GIFelement *s,struct GIFelement *e, FILE *fp, int lastimg)
 */
 #if 0
 	case GIFplaintext:
-	    data = access_data(s, 0, s->size);
 	    /* gIFt is 12 bytes longer than GCE, due to 32-bit ints and
 	       rgb colors */
 	    png_write_chunk_start(png_ptr, "gIFt", s->size+12);
 	    memset(buffer, 0, 24);
-	    buffer[2] = data[1];
-	    buffer[3] = data[0];
-	    buffer[6] = data[3];
-	    buffer[7] = data[2];
-	    buffer[10] = data[5];
-	    buffer[11] = data[4];
-	    buffer[14] = data[7];
-	    buffer[15] = data[6];
-	    buffer[16] = data[8];
-	    buffer[17] = data[9];
-	    buffer[18] = GifScreen.ColorMap[data[10]].red;
-	    buffer[19] = GifScreen.ColorMap[data[10]].green;
-	    buffer[20] = GifScreen.ColorMap[data[10]].blue;
-	    buffer[21] = GifScreen.ColorMap[data[11]].red;
-	    buffer[22] = GifScreen.ColorMap[data[11]].green;
-	    buffer[23] = GifScreen.ColorMap[data[11]].blue;
-	    png_write_chunk_data(png_ptr, buffer, 24);
-	    png_write_chunk_data(png_ptr, data+12, s->size-12);
+	    buffer[2] = s->data[1];
+	    buffer[3] = s->data[0];
+	    buffer[6] = s->data[3];
+	    buffer[7] = s->data[2];
+	    buffer[10] = s->data[5];
+	    buffer[11] = s->data[4];
+	    buffer[14] = s->data[7];
+	    buffer[15] = s->data[6];
+	    buffer[16] = s->data[8];
+	    buffer[17] = s->data[9];
+	    buffer[18] = GifScreen.ColorMap[s->data[10]].red;
+	    buffer[19] = GifScreen.ColorMap[s->data[10]].green;
+	    buffer[20] = GifScreen.ColorMap[s->data[10]].blue;
+	    buffer[21] = GifScreen.ColorMap[s->data[11]].red;
+	    buffer[22] = GifScreen.ColorMap[s->data[11]].green;
+	    buffer[23] = GifScreen.ColorMap[s->data[11]].blue;
+	    png_write_chunk_s->data(png_ptr, buffer, 24);
+	    png_write_chunk_s->data(png_ptr, s->data+12, s->size-12);
 	    png_write_chunk_end(png_ptr);
 	    break;
 #endif
 
 	case GIFgraphicctl:
-	    data = access_data(s, 0, s->size);
-	    buffer[0] = (data[0] >> 2) & 7;
-	    buffer[1] = (data[0] >> 1) & 1;
-	    buffer[2] = data[2];
-	    buffer[3] = data[1];
+	    buffer[0] = (s->data[0] >> 2) & 7;
+	    buffer[1] = (s->data[0] >> 1) & 1;
+	    buffer[2] = s->data[2];
+	    buffer[3] = s->data[1];
 
 	    png_write_chunk(png_ptr, (unsigned char *)"gIFg", buffer, 4);
 	    break;
@@ -657,10 +644,6 @@ int processfile(char *fname, FILE *fp)
 
     if (fp == NULL) return 1;
 
-#ifdef TMPFILE
-    fseek(tempfile, 0, SEEK_SET);
-#endif
-
     current = &first;
 
     num_pics = ReadGIF (fp);
@@ -876,13 +859,6 @@ int main(int argc, char *argv[])
     if (verbose > 1)
 	fprintf(stderr, "gif2png: %s, %s\n", version, compile_info);
 
-#ifdef TMPFILE
-    if ((tempfile = tmpfile()) == NULL) {
-	perror("tempfile");
-	exit(1);
-    }
-#endif
-
     /* loop over arguments or use stdin */
     if (argc == ac) {
 	if (input_is_terminal()) {
