@@ -32,12 +32,8 @@ struct GIFimagestruct {
 struct GIFelement {
   struct GIFelement *next;
   char GIFtype;
-#ifndef TMPFILE
   byte *data;
   long allocated_size;
-#else
-  unsigned long file_offset;
-#endif
   long size;
   /* only used if GIFtype==GIFimage */
   struct GIFimagestruct *imagestruct;
@@ -65,7 +61,6 @@ void set_size(long);
 void *xalloc(unsigned long s);
 void *xrealloc(void *p, unsigned long s);
 void fix_current(void);
-byte *access_data(struct GIFelement *e, unsigned long pos, unsigned long len);
 void free_mem(void);
 
 int interlace_line(int height, int line);
@@ -75,10 +70,6 @@ extern struct GIFelement first;
 extern struct GIFelement *current;
 extern int recover;
 
-#ifdef TMPFILE
-extern FILE *tempfile;
-#endif
-
 extern const char version[];
 extern const char compile_info[];
 
