@@ -596,23 +596,12 @@ get_prev_line(int width, int height, int line)
 {
   int prev_line;
   byte *res;
-#ifdef TMPFILE
-  long filepos;
-#endif
 
   prev_line=inv_interlace_line(height,line)-1;
   while(interlace_line(height, prev_line)>=line)
     prev_line--;
 
-#ifdef TMPFILE
-  filepos=ftell(tempfile);
-#endif
-
-  res=access_data(current, width*interlace_line(height, prev_line), width);
-
-#ifdef TMPFILE
-  fseek(tempfile, filepos, SEEK_SET);
-#endif
+  res = current->data + width*interlace_line(height, prev_line);
 
   return res;
 }
